package xyz.pixelatedw.finalbeta.mixin;

import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;

import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.level.Level;
import net.minecraft.tile.FenceTile;
import net.minecraft.tile.Tile;
import net.minecraft.tile.material.Material;
import net.minecraft.util.maths.Box;

@Mixin(FenceTile.class)
public class FenceTileMixin extends Tile {

	public FenceTileMixin(int i, Material j) {
		super(i, j);
	}

	@Inject(method = "canPlaceAt", at = @At("HEAD"), cancellable = true)
	public void canPlaceAt(Level arg, int i, int j, int k, CallbackInfoReturnable<Boolean> cir) {
		cir.setReturnValue(true);
	}

	@Inject(method = "getCollisionShape", at = @At("HEAD"), cancellable = true)
	public void getCollisionShape(Level level, int x, int y, int z, CallbackInfoReturnable<Box> cir) {
		cir.setReturnValue(createFenceBox(level, x, y, z, false));
	}

	@Override
	@Environment(EnvType.CLIENT)
	public Box getOutlineShape(Level level, int x, int y, int z) {
		return createFenceBox(level, x, y, z, true);
	}

	private Box createFenceBox(Level level, int x, int y, int z, boolean isOutline) {
		Tile tile = ((FenceTile)(Object)this);
		
		Tile xpTile = Tile.BY_ID[level.getTileId(x + 1, y, z)];
		Tile xnTile = Tile.BY_ID[level.getTileId(x - 1, y, z)];
		Tile zpTile = Tile.BY_ID[level.getTileId(x, y, z + 1)];
		Tile znTile = Tile.BY_ID[level.getTileId(x, y, z - 1)];
		
		boolean xpCheck = xpTile != null && (xpTile.isFullCube() || xpTile.id == Tile.FENCE.id);
		boolean xnCheck = xnTile != null && (xnTile.isFullCube() || xnTile.id == Tile.FENCE.id);
		boolean zpCheck = zpTile != null && (zpTile.isFullCube() || zpTile.id == Tile.FENCE.id);
		boolean znCheck = znTile != null && (znTile.isFullCube() || znTile.id == Tile.FENCE.id);
			
		boolean eastCheck = level.getMaterial(x + 1, y, z).isSolid() && xpCheck;
		boolean westCheck = level.getMaterial(x - 1, y, z).isSolid() && xnCheck;
		boolean southCheck = level.getMaterial(x, y, z + 1).isSolid() && zpCheck;
		boolean northCheck = level.getMaterial(x, y, z - 1).isSolid() && znCheck;
		
		Box box = Box.create(westCheck ? 0 : 0.375f, 0.0f, northCheck ? 0.0f : 0.375f, eastCheck ? 1.0f : 0.625f, 1.0F, southCheck ? 1.0f : 0.625f);
		
		tile.minX = westCheck ? 0 : 0.375f;
		tile.minY = 0.0;
		tile.minZ = northCheck ? 0.0f : 0.375f;
		tile.maxX = eastCheck ? 1.0f : 0.625f;
		tile.maxY = 1.0f;
		tile.maxZ = southCheck ? 1.0f : 0.625f;
		
        box.minX += x;
        box.minY += y;
        box.minZ += z;
        box.maxX += x;
        box.maxY += y;
        box.maxZ += z;
		
		if (!isOutline) {
			box.maxY += 0.5f;
		}

		return box;
	}
}
